package javax.management.timer;

import java.util.ArrayList;
import java.util.Vector;
import javax.management.InstanceNotFoundException;
import java.util.Date;
import java.util.Iterator;
import java.util.TreeSet;
import javax.management.MBeanNotificationInfo;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.ObjectName;
import javax.management.MBeanServer;
import java.util.HashMap;
import java.util.Map;
import javax.management.MBeanRegistration;
import javax.management.NotificationBroadcasterSupport;

public class Timer extends NotificationBroadcasterSupport implements TimerMBean, MBeanRegistration
{
    public static final long ONE_SECOND = 1000L;
    public static final long ONE_MINUTE = 60000L;
    public static final long ONE_HOUR = 3600000L;
    public static final long ONE_DAY = 86400000L;
    public static final long ONE_WEEK = 604800000L;
    private final Map<Integer, Object[]> timerTable;
    private boolean sendPastNotifications;
    private transient boolean isActive;
    private transient long sequenceNumber;
    private static final int TIMER_NOTIF_INDEX = 0;
    private static final int TIMER_DATE_INDEX = 1;
    private static final int TIMER_PERIOD_INDEX = 2;
    private static final int TIMER_NB_OCCUR_INDEX = 3;
    private static final int ALARM_CLOCK_INDEX = 4;
    private static final int FIXED_RATE_INDEX = 5;
    private volatile int counterID;
    private java.util.Timer timer;
    
    public Timer() {
        this.timerTable = new HashMap();
        this.sendPastNotifications = false;
        this.isActive = false;
        this.sequenceNumber = 0L;
        this.counterID = 0;
    }
    
    @Override
    public ObjectName preRegister(MBeanServer mBeanServer, ObjectName objectName) throws Exception {
        return objectName;
    }
    
    @Override
    public void postRegister(Boolean b) {
    }
    
    @Override
    public void preDeregister() throws Exception {
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "preDeregister", "stop the timer");
        this.stop();
    }
    
    @Override
    public void postDeregister() {
    }
    
    @Override
    public synchronized MBeanNotificationInfo[] getNotificationInfo() {
        TreeSet<String> set = new TreeSet();
        Iterator<Object> iterator = this.timerTable.values().iterator();
        while (iterator.hasNext()) {
            set.add(((TimerNotification)((Object[])iterator.next())[0]).getType());
        }
        return new MBeanNotificationInfo[] { new MBeanNotificationInfo((String[])set.toArray(new String[0]), TimerNotification.class.getName(), "Notification sent by Timer MBean") };
    }
    
    @Override
    public synchronized void start() {
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "start", "starting the timer");
        if (!this.isActive) {
            this.timer = new java.util.Timer();
            Date date = new Date();
            this.sendPastNotifications(date, this.sendPastNotifications);
            for (Object[] array : this.timerTable.values()) {
                Date date2 = (Date)array[1];
                if ((boolean)array[5]) {
                    TimerAlarmClock task2 = new TimerAlarmClock(this, date2);
                    array[4] = task2;
                    this.timer.schedule(task2, task2.next);
                }
                else {
                    TimerAlarmClock task = new TimerAlarmClock(this, date2.getTime() - date.getTime());
                    array[4] = task;
                    this.timer.schedule(task, task.timeout);
                }
            }
            this.isActive = true;
            JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "start", "timer started");
        }
        else {
            JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "start", "the timer is already activated");
        }
    }
    
    @Override
    public synchronized void stop() {
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "stop", "stopping the timer");
        if (this.isActive) {
            Iterator<Object> iterator = this.timerTable.values().iterator();
            while (iterator.hasNext()) {
                TimerAlarmClock timerAlarmClock = (TimerAlarmClock)((Object[])iterator.next())[4];
                if (timerAlarmClock != null) {
                    timerAlarmClock.cancel();
                }
                else {
                    continue;
                }
            }
            this.timer.cancel();
            this.isActive = false;
            JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "stop", "timer stopped");
        }
        else {
            JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "stop", "the timer is already deactivated");
        }
    }
    
    @Override
    public synchronized Integer addNotification(String s, String s2, Object userData, Date date, long n, long n2, boolean b) throws IllegalArgumentException {
        if (date == null) {
            throw new IllegalArgumentException("Timer notification date cannot be null.");
        }
        if (n < 0L || n2 < 0L) {
            throw new IllegalArgumentException("Negative values for the periodicity");
        }
        Date date2 = new Date();
        if (date2.after(date)) {
            date.setTime(date2.getTime());
            if (JmxProperties.TIMER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "addNotification", "update timer notification to add with:\n\tNotification date = " + date);
            }
        }
        Integer value = Integer.valueOf(++this.counterID);
        TimerNotification timerNotification = new TimerNotification(s, this, 0L, 0L, s2, value);
        timerNotification.setUserData(userData);
        Object[] array = new Object[6];
        TimerAlarmClock timerAlarmClock;
        if (b) {
            timerAlarmClock = new TimerAlarmClock(this, date);
        }
        else {
            timerAlarmClock = new TimerAlarmClock(this, date.getTime() - date2.getTime());
        }
        Date obj = new Date(date.getTime());
        array[0] = timerNotification;
        array[1] = obj;
        array[2] = Long.valueOf(n);
        array[3] = Long.valueOf(n2);
        array[4] = timerAlarmClock;
        array[5] = Boolean.valueOf(b);
        if (JmxProperties.TIMER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "addNotification", "adding timer notification:\n\t" + "Notification source = " + timerNotification.getSource() + "\n\tNotification type = " + timerNotification.getType() + "\n\tNotification ID = " + value + "\n\tNotification date = " + obj + "\n\tNotification period = " + n + "\n\tNotification nb of occurrences = " + n2 + "\n\tNotification executes at fixed rate = " + b);
        }
        this.timerTable.put(value, array);
        if (this.isActive) {
            if (b) {
                this.timer.schedule(timerAlarmClock, timerAlarmClock.next);
            }
            else {
                this.timer.schedule(timerAlarmClock, timerAlarmClock.timeout);
            }
        }
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "addNotification", "timer notification added");
        return value;
    }
    
    @Override
    public synchronized Integer addNotification(String s, String s2, Object o, Date date, long n, long n2) throws IllegalArgumentException {
        return this.addNotification(s, s2, o, date, n, n2, false);
    }
    
    @Override
    public synchronized Integer addNotification(String s, String s2, Object o, Date date, long n) throws IllegalArgumentException {
        return this.addNotification(s, s2, o, date, n, 0L);
    }
    
    @Override
    public synchronized Integer addNotification(String s, String s2, Object o, Date date) throws IllegalArgumentException {
        return this.addNotification(s, s2, o, date, 0L, 0L);
    }
    
    @Override
    public synchronized void removeNotification(Integer n) throws InstanceNotFoundException {
        if (!this.timerTable.containsKey(n)) {
            throw new InstanceNotFoundException("Timer notification to remove not in the list of notifications");
        }
        Object[] array = (Object[])this.timerTable.get(n);
        TimerAlarmClock timerAlarmClock = (TimerAlarmClock)array[4];
        if (timerAlarmClock != null) {
            timerAlarmClock.cancel();
        }
        if (JmxProperties.TIMER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "removeNotification", "removing timer notification:" + "\n\tNotification source = " + ((TimerNotification)array[0]).getSource() + "\n\tNotification type = " + ((TimerNotification)array[0]).getType() + "\n\tNotification ID = " + ((TimerNotification)array[0]).getNotificationID() + "\n\tNotification date = " + array[1] + "\n\tNotification period = " + array[2] + "\n\tNotification nb of occurrences = " + array[3] + "\n\tNotification executes at fixed rate = " + array[5]);
        }
        this.timerTable.remove(n);
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "removeNotification", "timer notification removed");
        return;
    }
    
    @Override
    public synchronized void removeNotifications(String s) throws InstanceNotFoundException {
        Vector<Integer> notificationIDs = this.getNotificationIDs(s);
        if (notificationIDs.isEmpty()) {
            throw new InstanceNotFoundException("Timer notifications to remove not in the list of notifications");
        }
        Iterator<Object> iterator = notificationIDs.iterator();
        while (iterator.hasNext()) {
            this.removeNotification((Integer)iterator.next());
        }
        return;
    }
    
    @Override
    public synchronized void removeAllNotifications() {
        Iterator<Object> iterator = this.timerTable.values().iterator();
        while (iterator.hasNext()) {
            ((TimerAlarmClock)((Object[])iterator.next())[4]).cancel();
        }
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "removeAllNotifications", "removing all timer notifications");
        this.timerTable.clear();
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "removeAllNotifications", "all timer notifications removed");
        this.counterID = 0;
        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "removeAllNotifications", "timer notification counter ID reset");
    }
    
    @Override
    public synchronized int getNbNotifications() {
        return this.timerTable.size();
    }
    
    @Override
    public synchronized Vector<Integer> getAllNotificationIDs() {
        return new Vector(this.timerTable.keySet());
    }
    
    @Override
    public synchronized Vector<Integer> getNotificationIDs(String s) {
        Vector<Object> vector = new Vector();
        for (Map.Entry entry : this.timerTable.entrySet()) {
            String type = ((TimerNotification)((Object[])entry.getValue())[0]).getType();
            if (!((s != null) ? (!s.equals(type)) : (type != null))) {
                vector.addElement(entry.getKey());
            }
            else {
                continue;
            }
        }
        return vector;
    }
    
    @Override
    public synchronized String getNotificationType(Integer n) {
        Object[] array = (Object[])this.timerTable.get(n);
        if (array != null) {
            return ((TimerNotification)array[0]).getType();
        }
        return null;
    }
    
    @Override
    public synchronized String getNotificationMessage(Integer n) {
        Object[] array = (Object[])this.timerTable.get(n);
        if (array != null) {
            return ((TimerNotification)array[0]).getMessage();
        }
        return null;
    }
    
    @Override
    public synchronized Object getNotificationUserData(Integer n) {
        Object[] array = (Object[])this.timerTable.get(n);
        if (array != null) {
            return ((TimerNotification)array[0]).getUserData();
        }
        return null;
    }
    
    @Override
    public synchronized Date getDate(Integer n) {
        Object[] array = (Object[])this.timerTable.get(n);
        if (array != null) {
            return new Date(((Date)array[1]).getTime());
        }
        return null;
    }
    
    @Override
    public synchronized Long getPeriod(Integer n) {
        Object[] array = (Object[])this.timerTable.get(n);
        if (array != null) {
            return (Long)array[2];
        }
        return null;
    }
    
    @Override
    public synchronized Long getNbOccurences(Integer n) {
        Object[] array = (Object[])this.timerTable.get(n);
        if (array != null) {
            return (Long)array[3];
        }
        return null;
    }
    
    @Override
    public synchronized Boolean getFixedRate(Integer n) {
        Object[] array = (Object[])this.timerTable.get(n);
        if (array != null) {
            return Boolean.valueOf((boolean)array[5]);
        }
        return null;
    }
    
    @Override
    public boolean getSendPastNotifications() {
        return this.sendPastNotifications;
    }
    
    @Override
    public void setSendPastNotifications(boolean sendPastNotifications) {
        this.sendPastNotifications = sendPastNotifications;
    }
    
    @Override
    public boolean isActive() {
        return this.isActive;
    }
    
    @Override
    public synchronized boolean isEmpty() {
        return this.timerTable.isEmpty();
    }
    
    private synchronized void sendPastNotifications(Date date, boolean b) {
        for (Object[] array : new ArrayList(this.timerTable.values())) {
            TimerNotification timerNotification = (TimerNotification)array[0];
            Integer notificationID = timerNotification.getNotificationID();
            Date date2 = (Date)array[1];
            while (date.after(date2) && this.timerTable.containsKey(notificationID)) {
                if (b) {
                    if (JmxProperties.TIMER_LOGGER.isLoggable(Level.FINER)) {
                        JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "sendPastNotifications", "sending past timer notification:" + "\n\tNotification source = " + timerNotification.getSource() + "\n\tNotification type = " + timerNotification.getType() + "\n\tNotification ID = " + timerNotification.getNotificationID() + "\n\tNotification date = " + date2 + "\n\tNotification period = " + array[2] + "\n\tNotification nb of occurrences = " + array[3] + "\n\tNotification executes at fixed rate = " + array[5]);
                    }
                    this.sendNotification(date2, timerNotification);
                    JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "sendPastNotifications", "past timer notification sent");
                }
                this.updateTimerTable(timerNotification.getNotificationID());
            }
        }
    }
    
    private synchronized void updateTimerTable(Integer obj) {
        Object[] array = (Object[])this.timerTable.get(obj);
        Date obj2 = (Date)array[1];
        Long obj3 = (Long)array[2];
        Long n = (Long)array[3];
        Boolean obj4 = (Boolean)array[5];
        TimerAlarmClock timerAlarmClock = (TimerAlarmClock)array[4];
        if ((long)obj3 != 0L) {
            if ((long)n == 0L || (long)n > 1L) {
                obj2.setTime(obj2.getTime() + (long)obj3);
                array[3] = Long.valueOf(Math.max(0L, (long)n - 1L));
                Long obj5 = (Long)array[3];
                if (this.isActive) {
                    if ((boolean)obj4) {
                        TimerAlarmClock task2 = new TimerAlarmClock(this, obj2);
                        array[4] = task2;
                        this.timer.schedule(task2, task2.next);
                    }
                    else {
                        TimerAlarmClock task = new TimerAlarmClock(this, (long)obj3);
                        array[4] = task;
                        this.timer.schedule(task, task.timeout);
                    }
                }
                if (JmxProperties.TIMER_LOGGER.isLoggable(Level.FINER)) {
                    TimerNotification timerNotification = (TimerNotification)array[0];
                    JmxProperties.TIMER_LOGGER.logp(Level.FINER, Timer.class.getName(), "updateTimerTable", "update timer notification with:" + "\n\tNotification source = " + timerNotification.getSource() + "\n\tNotification type = " + timerNotification.getType() + "\n\tNotification ID = " + obj + "\n\tNotification date = " + obj2 + "\n\tNotification period = " + obj3 + "\n\tNotification nb of occurrences = " + obj5 + "\n\tNotification executes at fixed rate = " + obj4);
                }
            }
            else {
                if (timerAlarmClock != null) {
                    timerAlarmClock.cancel();
                }
                this.timerTable.remove(obj);
            }
        }
        else {
            if (timerAlarmClock != null) {
                timerAlarmClock.cancel();
            }
            this.timerTable.remove(obj);
        }
    }
    
    void notifyAlarmClock(TimerAlarmClockNotification timerAlarmClockNotification) {
        TimerNotification timerNotification = null;
        Date date = null;
        TimerAlarmClock timerAlarmClock = (TimerAlarmClock)timerAlarmClockNotification.getSource();
        synchronized (this) {
            for (Object[] array : this.timerTable.values()) {
                if (array[4] == timerAlarmClock) {
                    timerNotification = (TimerNotification)array[0];
                    date = (Date)array[1];
                    break;
                }
                continue;
            }
        }
        this.sendNotification(date, timerNotification);
        this.updateTimerTable(timerNotification.getNotificationID());
    }
    
    void sendNotification(Date obj, TimerNotification timerNotification) {
        // Bytecode:
        // 
        //     6: invokevirtual   java/util/logging/Logger.isLoggable:(Ljava/util/logging/Level;)Z
        //     9: ifeq            90
        //    12: new             Ljava/lang/StringBuilder;
        //    15: dup            
        //    16: invokespecial   java/lang/StringBuilder.<init>:()V
        //    19: ldc             "sending timer notification:"
        //    21: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    24: ldc             "\n\tNotification source = "
        //    26: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    29: aload_2        
        //    30: invokevirtual   javax/management/timer/TimerNotification.getSource:()Ljava/lang/Object;
        //    33: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/Object;)Ljava/lang/StringBuilder;
        //    36: ldc             "\n\tNotification type = "
        //    38: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    41: aload_2        
        //    42: invokevirtual   javax/management/timer/TimerNotification.getType:()Ljava/lang/String;
        //    45: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    48: ldc             "\n\tNotification ID = "
        //    50: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    53: aload_2        
        //    54: invokevirtual   javax/management/timer/TimerNotification.getNotificationID:()Ljava/lang/Integer;
        //    57: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/Object;)Ljava/lang/StringBuilder;
        //    60: ldc             "\n\tNotification date = "
        //    62: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    65: aload_1        
        //    66: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/Object;)Ljava/lang/StringBuilder;
        //    69: astore_3       
        //    70: getstatic       com/sun/jmx/defaults/JmxProperties.TIMER_LOGGER:Ljava/util/logging/Logger;
        //    73: getstatic       java/util/logging/Level.FINER:Ljava/util/logging/Level;
        //    76: ldc             Ljavax/management/timer/Timer;.class
        //    78: invokevirtual   java/lang/Class.getName:()Ljava/lang/String;
        //    81: ldc             "sendNotification"
        //    83: aload_3        
        //    84: invokevirtual   java/lang/StringBuilder.toString:()Ljava/lang/String;
        //    87: invokevirtual   java/util/logging/Logger.logp:(Ljava/util/logging/Level;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
        //    90: aload_0        
        //    91: dup            
        //    92: astore          5
        //    94: monitorenter   
        //    95: aload_0        
        //    96: aload_0        
        //    97: getfield        javax/management/timer/Timer.sequenceNumber:J
        //   100: lconst_1       
        //   101: ladd           
        //   102: putfield        javax/management/timer/Timer.sequenceNumber:J
        //   105: aload_0        
        //   106: getfield        javax/management/timer/Timer.sequenceNumber:J
        //   109: lstore_3       
        //   110: aload           5
        //   112: monitorexit    
        //   113: goto            124
        //   116: astore          6
        //   118: aload           5
        //   120: monitorexit    
        //   121: aload           6
        //   123: athrow         
        //   124: aload_2        
        //   125: dup            
        //   126: astore          5
        //   128: monitorenter   
        //   129: aload_2        
        //   130: aload_1        
        //   131: invokevirtual   java/util/Date.getTime:()J
        //   134: invokevirtual   javax/management/timer/TimerNotification.setTimeStamp:(J)V
        //   137: aload_2        
        //   138: lload_3        
        //   139: invokevirtual   javax/management/timer/TimerNotification.setSequenceNumber:(J)V
        //   142: aload_0        
        //   143: aload_2        
        //   144: invokevirtual   javax/management/timer/TimerNotification.cloneTimerNotification:()Ljava/lang/Object;
        //   147: checkcast       Ljavax/management/timer/TimerNotification;
        //   150: invokevirtual   javax/management/timer/Timer.sendNotification:(Ljavax/management/Notification;)V
        //   153: aload           5
        //   155: monitorexit    
        //   156: goto            167
        //   159: astore          7
        //   161: aload           5
        //   163: monitorexit    
        //   164: aload           7
        //   166: athrow         
        //   167: getstatic       com/sun/jmx/defaults/JmxProperties.TIMER_LOGGER:Ljava/util/logging/Logger;
        //   170: getstatic       java/util/logging/Level.FINER:Ljava/util/logging/Level;
        //   173: ldc             Ljavax/management/timer/Timer;.class
        //   175: invokevirtual   java/lang/Class.getName:()Ljava/lang/String;
        //   178: ldc             "sendNotification"
        //   180: ldc             "timer notification sent"
        //   182: invokevirtual   java/util/logging/Logger.logp:(Ljava/util/logging/Level;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
        //   185: return         
        //    StackMapTable: 00 05 FB 00 5A FF 00 19 00 06 07 00 2B 07 01 04 07 00 2C 00 00 07 00 FE 00 01 07 01 01 FF 00 07 00 04 07 00 2B 07 01 04 07 00 2C 04 00 00 FF 00 22 00 05 07 00 2B 07 01 04 07 00 2C 04 07 00 FE 00 01 07 01 01 FA 00 07
        //    Exceptions:
        //  Try           Handler
        //  Start  End    Start  End    Type
        //  -----  -----  -----  -----  ----
        //  95     113    116    124    Any
        //  116    121    116    124    Any
        //  129    156    159    167    Any
        //  159    164    159    167    Any
        // 
        throw new IllegalStateException("An error occurred while decompiling this method.");
    }
}
